# PetaLinux for Scalp board

## Documentation to create a custom BSP for PetaLinux

PetaLinux is an extension of the Yocto project that provides official support (Xilinx layers) for Xilinx SoCs Arm (Zynq and ZynqMP) and Xilinx Microblaze.

**Please note that this workflow only works with Linux. Please consult the Linux distributions supported by Vivado. If you intend to use Windows and the Linux subsystem. Don't waste your time asking me questions.**

* [PetaLinux 2020.2 Package List](https://www.xilinx.com/Attachment/2020.2_PetaLinux_Package_List.xlsx)
* [PetaLinux 2020.2 user guide](https://www.xilinx.com/support/documentation/sw_manuals/xilinx2020_2/ug1144-petalinux-tools-reference-guide.pdf)

### First steps to perform

* Installing Vivado 2020.2
* Installing PetaLinux 2020.2
* Setting-Up your environnement

#### Setting-Up your environnement

Run the following command from a terminal.

```
$ source /<XILINX_INSTALL_FOLDER>/PetaLinux/2020.2/bin/settings.sh
```

In my case /<XILINX\_INSTALL\_FOLDER>/ corresponds to /tools/Xilinx/.

**Check that the "rsync" tool is installed.**

### Parts of a BSP (Board Support Package)

* Device-tree (U-Boot and Linux)
* Kernel configuration
* U-Boot configuration
* Boot script
* Possible patchs

### BSP creation workflow

* Create and export hardware design.
* Create a blank PetaLinux project without a BSP.
* Customization process to meet the requirements of the BSP.
* Package the PetaLinux project into a BSP.
* Test the BSP by running directly and by creating a new project.

### Clone of the GIT Scalp Firmware repository (HDL)

```
$ cd <SCALP_PROJECT_FOLDER>
$ git clone https://gitedu.hesge.ch/soma/scalp_firmware.git
$ cd scalp_firmware
$ git checkout SCALP-SAFE-v0.01
```

### FPGA scalp_safe_firmware creation

```
$ cd <SCALP_PROJECT_FOLDER>/scalp_firmware/designs/vivado/scalp_safe_firmware/2020.2/lin64/
$ source setup.sh
$ create_project
$ gen_bitstream
$ export_hw
or
$ open_gui
and File -> Export -> Export Hardware -> include bitstream
```

### Clone of the GIT Scalp PetaLinux repository

```
$ cd <SCALP_PROJECT_FOLDER>
$ git clone https://gitedu.hesge.ch/soma/scalp_petalinux.git
$ cd scalp_petalinux
$ git checkout SCALP-SAFE-v0.01
```

### Create new Petalinux project from a BSP

```
$ cd <SCALP_PROJECT_FOLDER>/scalp_petalinux/
$ petalinux-create --type project -s <path-to-bsp> --name <PROJECT NAME>
$ cd <PROJECT NAME>
```

In my case **<path-to-bsp>** corresponds to /home/jo/Documents/Projets/Hepia/scalp_project/scalp_petalinux/bsp/SCALP-SAFE-FIRMWARE-BSP-V0.01.BSP.bsp

### Importing hardware configuration

**The hardware description of the project must have been exported from Vivado beforehand.**

```
$ petalinux-config --get-hw-description=<path-to-hw> --silentconfig
```

In my case **<path-to-hw>** corresponds to /home/jo/Documents/Projets/Hepia/scalp_project/scalp_firmware/designs/vivado/scalp_safe_firmware/2020.2/lin64/scalp_safe_firmware/scalp_safe_firmware.xsa

#### Directory tree

```
<PROJECT NAME>
├── components
│   └── plnx_workspace
│       └── device-tree
│           └── device-tree
│               └── include
│                   └── dt-bindings
│                       ├── clock
│                       ├── power
│                       └── reset
├── hardware
│   └── scalp_safe_firmware
│       ├── scalp_safe_firmware.cache
│       │   ├── compile_simlib
│       │   │   ├── activehdl
│       │   │   ├── ies
│       │   │   ├── modelsim
│       │   │   ├── questa
│       │   │   ├── riviera
│       │   │   ├── vcs
│       │   │   └── xcelium
│       │   ├── ip
│       │   │   └── 2020.2
│       │   │       ├── 0aa2a8d458771a97
│       │   │       ├── 0bbb37acf8a11f47
│       │   │       ├── 129637f02d4c08f6
│       │   │       ├── 13ec2d6b4b16e74c
│       │   │       ├── 1b48bca4c59faa2a
│       │   │       ├── 2ddf55dbeb806356
│       │   │       ├── 72c991d9c4c4299d
│       │   │       └── 9a7eae85eedde786
│       │   └── wt
│       ├── scalp_safe_firmware.hw
│       ├── scalp_safe_firmware.ip_user_files
│       ├── scalp_safe_firmware.runs
│       │   ├── impl_1
│       │   ├── scalp_zynqps_auto_pc_0_synth_1
│       │   ├── scalp_zynqps_processing_system7_0_0_synth_1
│       │   ├── scalp_zynqps_rst_ps7_0_125M_0_synth_1
│       │   ├── scalp_zynqps_scalp_axi4lite_0_0_synth_1
│       │   ├── scalp_zynqps_util_vector_logic_0_0_synth_1
│       │   ├── scalp_zynqps_util_vector_logic_1_0_synth_1
│       │   ├── scalp_zynqps_vio_0_0_synth_1
│       │   └── synth_1
│       └── scalp_safe_firmware.sim
└── project-spec
    ├── configs
    │   ├── busybox
    │   ├── init-ifupdown
    │   └── u-boot-xlnx
    ├── hw-description
    └── meta-user
        ├── conf
        ├── recipes-apps
        │   ├── gpio-demo
        │   │   └── files
        │   └── peekpoke
        │       └── files
        ├── recipes-bsp
        │   ├── device-tree
        │   │   └── files
        │   ├── fsbl
        │   └── u-boot
        │       ├── files
        │       └── u-boot-zynq-scr
        └── recipes-kernel
            └── linux
                └── linux-xlnx

```

### Build the PetaLinux project

```
$ petalinux-build
```

The produced binary files can be found in the **./images/linux** folder.

### QSPI firmware packaging

```
$ petalinux-package --boot --fsbl images/linux/zynq_fsbl.elf --fpga images/linux/system.bit --u-boot --kernel --offset 0x520000
```

The produced binary file can be found in the **./images/linux** folder. This is copied in **0x520000** offset in the final image.

### Flash firmware on QSPI memory

```
petalinux-package --boot --fsbl images/linux/zynq_fsbl.elf --fpga images/linux/system.bit --u-boot --kernel --offset 0x520000
```

The **image.ub** file corresponds to the image of the Linux kernel, the device tree and the RamFS file system in FIT format.

The image content can be viewed using the **dumpimage** command.

```
/tools/u-boot-tools/dumpimage -l image.ub
```

In my case, I copied the tools produced by U-Boot into the **/tools/u-boot-tools/** folder.
